import json
from os import path
from pathlib import Path  # if you haven't already done so

file = Path(__file__).resolve()
parent, root = file.parent, file.parents[1]


def read_config_file(filename):
    with open(path.join(parent, filename), "r") as f:
        config = json.load(f)
    return config


def write_config_file(config, filename):
    writeable_config = json.dumps(config, indent=4)
    with open(path.join(parent, filename), "w") as f:
        f.write(writeable_config)


def set_config(key, value, filename="config.json"):
    config = read_config_file(filename=filename)
    config[key] = value
    write_config_file(config, filename=filename)


def get_config(key, filename="config.json"):
    config = read_config_file(filename=filename)
    return config[key]


class Config(object):
    def __init__(self, filename):
        self.filename = filename

    def __getattr__(self, key):
        """对于要请求的参数 直接从配置文件获取
        Args:
            key (string): 文件名
        """
        return get_config(key, filename=self.filename)



#!/usr/bin/env python

import os
import unittest
from unittest import TestCase

import pytest

from lark_sdk import Lark, LarkSender

from .utils import *

# 需要的 config 信息
APP_ID = "APP_ID"
APP_SECRET = "SECRET"
USER_EMAIL = "EMAIL"
OPEN_ID = "OPEN_ID"


class LarkUtilTestCase(TestCase):
    def setUp(self):
        self.APP_ID = get_config(APP_ID)
        self.SECRET = get_config(APP_SECRET)

    def test_lark_sdk_init(self):
        """测试初始化
        """
        lark = Lark(self.APP_ID, self.SECRET)
        self.assertTrue(lark.app_access_token)


class larkSenderTest(TestCase):
    def setUp(self):
        self.APP_ID = get_config(APP_ID)
        self.SECRET = get_config(APP_SECRET)
        self.EMAIL = get_config(USER_EMAIL)
        self.open_ids = [get_config(OPEN_ID)]
        self.lark_sender = LarkSender(self.APP_ID, self.SECRET, {"email": self.EMAIL})
        self.message_id = self.lark_sender.send_text("Hey")["message_id"]

    def test_lark_send_test_to_email(self):
        response = self.lark_sender.send_text("Hey")
        self.assertTrue(response["message_id"])

    def test_lark_send_post_to_email(self):
        response = self.lark_sender.send_post(
            "Hey", [[{"tag": "text", "un_escape": True, "text": "Hey&nbsp;"}]]
        )
        self.assertTrue(response["message_id"])

    def test_lark_read_info(self):
        read_info = self.lark_sender.read_info(self.message_id)
        self.assertTrue(read_info)

    def test_lark_recall(self):
        response = self.lark_sender.recall_message(self.message_id)
        self.assertTrue(response["code"] == 0)

    def test_lark_urgent(self):
        lark_sender = LarkSender(
            self.APP_ID, self.SECRET, {"email": self.EMAIL, "open_ids": self.open_ids}
        )
        message = lark_sender.urgent(self.message_id)
        self.assertTrue(message)



# coding: utf-8
#!/usr/bin/env python
import json
import unittest
from os import path
from pathlib import Path  # if you haven't already done so
from unittest import TestCase

if __name__ == "__main__":
    import sys

    file = Path(__file__).resolve()
    parent, root = file.parent, file.parents[1]
    sys.path.append(str(root))
    from utils import Config, get_config, set_config
else:
    from .utils import Config, set_config, get_config

from lark_sdk import Lark, LarkDocs, random_string

config = Config("lark_docs.json")
# 需要的 config 信息
APP_ID = "APP_ID"
SECRET = "SECRET"
SHEET_TOKEN = "SHEET_TOKEN"
SHEET_RANGE = "SHEET_RANGE"
DOCS_REFRESH_TOKEN = "DOCS_REFRESH_TOKEN"
DOCS_ACCESS_TOKEN = "DOCS_REFRESH_TOKEN"


def get_app_access_token():
    lark = Lark(config.APP_ID, config.SECRET)
    return lark.get_attr("app_access_token")


def set_attr(name, value):
    set_config(name, value, filename="lark_docs.json")


def get_attr(name):
    return get_config(name, filename="lark_docs.json")


class LarkUtilTestCase(TestCase):
    def setUp(self):
        self.SHEET_TOKEN = config.SHEET_TOKEN
        self.SHEET_RANGE = config.SHEET_RANGE
        self.refresh_token = config.refresh_token
        self.access_token = config.access_token
        lark = Lark(config.APP_ID, config.SECRET)
        self.lark_docs = LarkDocs(
            self.access_token,
            self.refresh_token,
            get_attr=get_attr,
            set_attr=set_attr,
            app_access_token=get_app_access_token(),
            is_lasted=True,
        )

    def test_get_sheet(self):
        response = self.lark_docs.get_sheet_info(config.SHEET_TOKEN)
        self.assertTrue(response)

    def test_get_sheet_by_range(self):
        response = self.lark_docs.get_sheet_by_range(
            config.SHEET_TOKEN, config.SHEET_RANGE
        )
        self.assertTrue(response)

    def test_copy_docs(self):
        copy_data = {
            "type": "sheet",
            "dstFolderToken": config.TEMPLATE_FOLDER_TOKEN,
            "dstName": random_string(),
            "permissionNeeded": False,
            "CommentNeeded": False,
        }
        response = self.lark_docs.copy_docs(config.SHEET_TOKEN, copy_data)
        self.assertTrue(response)
        self.assertTrue(response["folderToken"] == config.TEMPLATE_FOLDER_TOKEN)

    def test_create_permissions(self):
        member = [
            {"member_type": "email", "member_id": config.OTHER_EMAIL, "perm": "view"}
        ]
        response = self.lark_docs.create_permissions(
            config.SHEET_TOKEN, "sheet", member
        )
        self.assertTrue(response["is_all_success"])

    def test_get_folder_children(self):
        response = self.lark_docs.get_folder_children(config.TEMPLATE_FOLDER_TOKEN)
        self.assertTrue("parentToken" in response)

    def test_write_sheet_by_more_range(self):
        response = self.lark_docs.write_sheet_by_more_range(
            config.SHEET_TOKEN,
            [{"range": config.SHEET_ID + "!A1:B2", "values": [[1, 2]],}],
        )
        self.assertTrue(response)

    def test_lock_sheet(self):
        response = self.lark_docs.lock_sheet(config.SHEET_TOKEN, config.SHEET_ID)
        self.assertTrue(response)

    def test_unlock_sheet(self):
        response = self.lark_docs.unlock_sheet(config.SHEET_TOKEN, config.SHEET_ID)
        self.assertTrue(response)

    def test_get_root_folder_metadata(self):
        response = self.lark_docs.get_root_folder_metadata()
        self.assertTrue(response)


if __name__ == "__main__":
    # 生成登录回调链接
    app_id = config.APP_ID
    app_secret = config.SECRET
    lark = Lark(app_id, app_secret)
    print(lark.get_redirect_uri("http://localhost.cn:3000/test"))
    # 访问该链接后需要手动获取 code
    code = input("请输入 code: ")
    userinfo = lark.code2userinfo(code)
    docs_access_token = userinfo["access_token"]
    docs_refresh_token = userinfo["refresh_token"]
    set_config(access_token, docs_access_token, filename="lark_docs.json")
    set_config(refresh_token, docs_refresh_token, filename="lark_docs.json")



# coding=utf-8
import functools
import json
import random
import sys
from string import ascii_uppercase
from time import time

if sys.version_info < (3, 1):
    from urllib import urlencode
else:
    from urllib.parse import urlencode

import requests

from .lib import post_request, pre_request


def random_string():
    return "".join(
        [random.choice(ascii_uppercase) for _ in range(random.randint(2, 6))]
    )


class getTokenError(Exception):
    pass


class LarkError(Exception):
    pass


class LarkRequestError(LarkError):
    pass


class LarkValueError(LarkError):
    pass


class LarkInitError(LarkError):
    pass


class InitTokenError(LarkError):
    pass


class Lark:
    """Lark Sdk 

    Raises:
        InitTokenError: 获取 Token 失败

    Returns:
        lark: lark 的基础实例 在这里完成获取 SDK 等任务
    """

    TOKEN_INIT_URL = (
        "https://open.feishu.cn/open-apis/auth/v3/app_access_token/internal/"
    )
    AUTH_STRING = "https://open.feishu.cn/open-apis/authen/v1/index"
    CALLBACK_URL = "http://retrace.bytedance.net/research/callback/"
    CODE2TOKEN_URL = "https://open.feishu.cn/open-apis/authen/v1/access_token"

    def set_attr(cls, key, value):
        setattr(cls, key, value)

    def get_attr(cls, key):
        return getattr(cls, key, None)

    def __init__(self, app_id, secret, set_attr=None, get_attr=None):
        """初始化 Lark 的 SDK 

        Args:
            app_id (string): app_id
            secret (string): app_secrete
            set_attr (func, optional): 给类绑定值，可以进行修改用于使用 redis 来创建变量. Defaults to None.
            get_attr (fucn, optional): 用于从实例中获取变量. Defaults to None.
        """
        self.set_attr = set_attr if set_attr else self.set_attr
        self.get_attr = get_attr if get_attr else self.get_attr
        # TODO: 使用 getter 和 setter 进行封装
        self.set_attr("app_id", app_id)
        self.set_attr("secret", secret)
        self.init_token()

    @staticmethod
    def data_parser(data, fields, use_dict=False):
        """用于数据过滤 避免存储不合规的数据

        Args:
            data (dict): 数据类型
            fields (list): 要保存的字段
            use_dict (bool): 返回的数据对象是否为 dict
        Return:
            data (list): value 的 列表 
        """
        return (
            {field: data[field] for field in fields}
            if use_dict
            else [data[field] for field in fields]
        )

    @pre_request
    def get(self, *args, **kwargs):
        return requests.get(*args, **kwargs)

    @pre_request
    def post(self, *args, **kwargs):
        return requests.post(*args, **kwargs)

    @pre_request
    def put(self, *args, **kwargs):
        return requests.put(*args, **kwargs)

    def init_token(self):
        """使用 APP_ID + SECRET 获取 Token

        Raises:
            getTokenError: 获取 用户信息 失败
        """
        response = self.post(
            self.TOKEN_INIT_URL, json={"app_id": self.app_id, "app_secret": self.secret}
        )
        if response["msg"] != "ok":
            raise getTokenError(response["msg"])
        app_access_token, expire = response["app_access_token"], response["expire"]
        self.set_attr("app_access_token", app_access_token)

    @post_request(LarkRequestError, parse_func=lambda response: response["bot"])
    def get_bot_info(self):
        """获取机器人自身信息
        https://open.feishu.cn/document/ukTMukTMukTM/uAjMxEjLwITMx4CMyETM
        
        Returns:
            Bot:{
                "activate_status": 2,
                "app_name": "name",
                "avatar_url": "https://sf1-ttcdn-tos.pstatp.com/obj/lark.avatar/da5xxxx14b16113",
                "ip_white_list": [],
                "open_id": "ou_xxx"  
            }
        """
        URL = "https://open.feishu.cn/open-apis/bot/v3/info/"
        return self.get(URL)

    def get_access_token(self):
        return self.get_attr("app_access_token")

    def code2userinfo(self, code):
        """使用 Code 获取用户信息

        Args:
            code (string): 回调获取到的 code

        Returns:
            userinfo: 获取的 userinfo 结构体 其中包含了 userToken，refreshToken，以及 openid 等用户信息

        userinfo: {
            "access_token": string,
            "en_name": string,
            "expires_in": int,
            "name": string,
            "open_id": string,
            "refresh_expires_in": int,
            "refresh_token": string,
            "tenant_key": string,
            "token_type": string
            "email": string
        }
        """
        response = self.post(
            self.CODE2TOKEN_URL,
            json={
                "app_access_token": self.get_access_token(),
                "grant_type": "authorization_code",
                "code": code,
            },
        )
        if response["code"] != 0:
            raise larkUtilError(response["msg"])
        return self.data_parser(
            response["data"],
            fields=[
                "access_token",
                "en_name",
                "name",
                "open_id",
                "expires_in",
                "refresh_token",
                "tenant_key",
            ],
            use_dict=True,
        )

    def get_redirect_uri(self, callback_url=None, state=None):
        """生成 SSO 登录回调链接 （ docs 需要）

        Args:
            callback_url (str, optional): callback_url. Defaults to None.
            state (str, optional): state. Defaults to None.

        Returns:
            uri: str,构造的回调链接
        """
        callback_url = callback_url if callback_url else self.CALLBACK_URL
        state = state if state else random_string()
        return (
            self.AUTH_STRING
            + "?"
            + urlencode(
                {"redirect_uri": callback_url, "app_id": self.app_id, "state": state,}
            )
        )



#!/usr/bin/env python

import os
import unittest
from unittest import TestCase

import pytest

from lark_sdk import Lark, LarkSender

from .utils import *

# 需要的 config 信息
APP_ID = "APP_ID"
APP_SECRET = "SECRET"
USER_EMAIL = "EMAIL"
OPEN_ID = "OPEN_ID"


class LarkUtilTestCase(TestCase):
    def setUp(self):
        self.APP_ID = get_config(APP_ID)
        self.SECRET = get_config(APP_SECRET)

    def test_lark_sdk_init(self):
        """测试初始化
        """
        lark = Lark(self.APP_ID, self.SECRET)
        self.assertTrue(lark.app_access_token)

    def test_lark_get_bot_info(self):
        lark = Lark(self.APP_ID, self.SECRET)
        bot_info = lark.get_bot_info()
        print(bot_info)
        self.assertTrue(bot_info)



import json


class LarkRequestError(Exception):
    pass


# coding=utf-8
def pre_request(func):
    def wrapper(self, *args, **kwargs):
        kwargs["headers"] = kwargs.get("headers", {"Content-Type": "application/json"})
        if self.get_access_token() and not kwargs["headers"].get("Authorization"):
            kwargs["headers"]["Authorization"] = "Bearer " + self.get_access_token()
        response = func(self, *args, **kwargs)
        if response.status_code == 200:
            return response.json()
        raise LarkRequestError(response.status_code, response.text)

    return wrapper


# 加打点
def post_request(
    exception, parse_func=lambda response: response["data"], valid_func=None
):
    """对请求结果进行再处理，验证参数是否请求成功（默认为检测 code 是否 为 0），如果不成功则抛出预定的报错，如果为成功，则执行默认的数据处理函数对数据进行在处理。

    Args:
        exception ([type]): [description]
        parse_func ([type], optional): [description]. Defaults to lambda response:response['data'].
        valid_func ([type], optional): [description]. Defaults to lambda response:response['code'] == 0.
    """

    def decorator(func):
        def response_check(response, exception, valid_func=None):
            """检验参数是否正确，默认使用 code = 0, 支持传入一个函数

            Args:
                response (any): 接受的 resposne
                exception (Exception): 出错时抛出的报错

            Raises:
                exception: 定义的报错
            """
            valid_func = (
                valid_func if valid_func else lambda response: response["code"] == 0
            )
            if not valid_func(response):
                raise exception(
                    response["message"]
                    if "message" in response
                    else json.dumps(response)
                )  # 抛出报错信息
            return response

        def wrapper(self, *args, **kwargs):
            response = response_check(
                func(self, *args, **kwargs), exception, valid_func
            )
            return parse_func(response)

        return wrapper

    return decorator



from .lark import *


class larkDockError(Exception):
    pass


class larkDocsSearchError(larkDockError):
    pass


class LarkDocs(Lark):
    SPREAD_SHEETS_TEMPLATE = "https://open.feishu.cn/open-apis/sheet/v2/spreadsheets/{SHEET_TOKEN}/values/{RANGE}"
    SHEETS_INFO = (
        "https://open.feishu.cn/open-apis/sheet/v2/spreadsheets/{SHEET_TOKEN}/metainfo"
    )
    DOCS_CREATE = (
        "https://open.feishu.cn/open-apis/drive/explorer/v2/file/{FOLDER_TOKEN}"
    )
    DOCS_COPY = "https://open.feishu.cn/open-apis/drive/explorer/v2/file/copy/files/{FILE_TOKEN}"
    DOCS_PERMISSION_CREATE = (
        "https://open.feishu.cn/open-apis/drive/permission/member/create"
    )
    FOLDER_CHILDREN = "https://open.feishu.cn/open-apis/drive/explorer/v2/folder/{FOLDER_TOKEN}/children"
    REFRESH_TOKEN = "https://open.feishu.cn/open-apis/authen/v1/refresh_access_token"

    def __init__(
        self,
        access_token,
        refresh_token=None,
        app_access_token=None,
        is_lasted=True,
        set_attr=None,
        get_attr=None,
    ):
        """初始化 SDK ， 如果需要实现刷新功能 Token 则需要填写 access_token 和 app_access_token，否则无法完成刷新
        https://open.feishu.cn/document/ukTMukTMukTM/uQDO4UjL0gDO14CN4gTN


        Args:
            access_token (str): [description]
            refresh_token (str, optional): [description]. Defaults to None.
            app_access_token (str, optional): 这个token 必须是 使用 code2token 时使用的 token . Defaults to None.
            is_lasted (bool, optional): 是否根据 refresh token 获取最新的 access Token. Defaults to True.
            set_attr (func, optional): 设置类的属性.
            get_attr (func, optional): 获取类的属性.
        """
        self.set_attr = set_attr if set_attr else self.set_attr
        self.get_attr = get_attr if get_attr else self.get_attr
        self.set_attr("access_token", access_token)
        self.set_attr("refresh_token", refresh_token)
        self.set_attr("app_access_token", app_access_token)
        self.set_attr("is_lasted", is_lasted)
        self.set_attr("refreshable", bool(refresh_token or app_access_token))

    def refresh_access_token(self):
        """https://open.feishu.cn/document/ukTMukTMukTM/uQDO4UjL0gDO14CN4gTN

        Raises:
            LarkVaildError: [description]
        """
        if not self.get_attr("refreshable"):
            raise LarkVaildError(u"缺少 refresh_token,app_access_token 无法完成刷新")
        data = self._post_refresh_token(
            {
                "app_access_token": self.get_attr("app_access_token"),
                "grant_type": "refresh_token",
                "refresh_token": self.get_attr("refresh_token"),
            }
        )
        access_token, refresh_token = self.data_parser(
            data, ["access_token", "refresh_token"]
        )
        self.set_attr("access_token", access_token)
        self.set_attr("refresh_token", refresh_token)

    @post_request(larkDockError)
    def _post_refresh_token(self, data):
        return requests.post(self.REFRESH_TOKEN, json=data).json()

    def get_access_token(self):
        if self.get_attr("is_lasted") and self.get_attr("refreshable"):
            self.refresh_access_token()
        return self.get_attr("access_token")

    @post_request(larkDockError)
    def get_sheet_info(self, sheet_token):
        """获取表格元数据

        Args:
            sheet_token (string):

        Returns:
            sheet_info: 表格信息
        {
            "properties": {
                "title": "",
                "ownerUser": 0,
                "sheetCount": 0,
                "revision": 0
            },
            "sheets": [
                {
                    "sheetId": "***",
                    "title": "***",
                    "index": 0,
                    "rowCount": 0,
                    "columnCount": 0
                }
            ],
            "spreadsheetToken": "***"
        }

        """
        url = self.SHEETS_INFO.format(**{"SHEET_TOKEN": sheet_token,})
        return self.get(url)

    @post_request(larkDockError)
    def create_docs(self, folder_token, title, docs_type="sheet"):
        """创建文档

        Args:
            folder_token (string): folder 的 token
            title (title): 文档的 Title
            docs_type (string, optional): 文件的类型. Defaults to "sheet".

        Returns:
            fileinfo: 文档的信息

        {
            "revision":0,
            "token":"string",
            "url":"string"
        }
        """
        url = DOCS_CREATE.format(**{"FOLDER_TOKEN": folder_token})
        return self.post(url, json={"title": title, "type": docs_type})

    @post_request(larkDockError)
    def copy_docs(self, file_token, docs_info):
        """对文档进行 copy
        https://open.feishu.cn/document/ugTM5UjL4ETO14COxkTN/uYTNzUjL2UzM14iN1MTN

        Args:
            file_token ([type]): file_token
            docs_info ([type]): 文档信息

                {
                    "type":"sheet",
                    "dstFolderToken":"string",
                    "dstName":"string",
                    "permissionNeeded":true,
                    "CommentNeeded":true
                }
        """
        url = self.DOCS_COPY.format(**{"FILE_TOKEN": file_token})
        return self.post(url, json=docs_info)

    @post_request(larkDockError)
    # WARAING: 无法授权给自己
    def create_permissions(self, file_token, docs_type, member):
        """给指定文件增加 权限

        Args:
            file_token (string): 文件的 token
            docs_type (string): 文件的类型
            member (string): 成员信息
                [
                    {
                        "member_type": "openid",
                        "member_id": "string",
                        "perm": "view"
                    }
                ]

        Returns:
            permissions: 权限信息

                "is_all_success": true,
                "fail_members": [
                    {
                        "member_type": "openid",
                        "member_id": "string",
                        "perm": "view"
                    }
                ]
        """
        return self.post(
            self.DOCS_PERMISSION_CREATE,
            json={"token": file_token, "type": docs_type, "members": member},
        )

    @post_request(larkDockError)
    def get_folder_children(self, folder_token, file_types=["sheet"]):
        """获取文件夹下的文档信息

        Args:
            folder_token (string)): 文件夹 Token
            file_types (list, optional): 文件种类可多选. Defaults to [].

        Returns:

            fils_infos: 文件信息
                {
                    "parentToken": "token",
                    "children": {
                        "lq3xxxxxwLfExB23": { //文件 token
                            "token": "lq3xxxxxwLfExB23",
                            "name": "test_folder_name",
                            "type": "folder",
                    },
                            "lq3xxxxxddddB24": { //文件 token
                                    "token": "lq3xxxxxddddB24",
                                    "name": "test_sheet_name",
                                    "type": "sheet",
                                }
                    }
                }
        """
        url = (
            self.FOLDER_CHILDREN.format(**{"FOLDER_TOKEN": folder_token})
            + "?"
            + "&".join(["type=" + file_type for file_type in file_types])
        )
        return self.get(url)

    @post_request(
        larkDockError,
        parse_func=lambda response: response["data"]["valueRange"]["values"],
    )
    def get_sheet_by_range(self, sheet_token, sheet_range):
        """获取指定的表格信息
        https://open.feishu.cn/document/ugTM5UjL4ETO14COxkTN/ugTMzUjL4EzM14COxMTN
        Args:
            sheet_token (string): sheet_token
            sheet_range (string): rage

        Raises:
            larkDocsSearchError: 获取失败

        Returns:
            sheetinfo: 获取到的表格信息

        """
        url = self.SPREAD_SHEETS_TEMPLATE.format(
            **{"SHEET_TOKEN": sheet_token, "RANGE": sheet_range,}
        )
        return self.get(url)

    @post_request(larkDockError)
    def write_sheet_by_range(self, sheet_token, sheet_range, value):
        url = self.SPREAD_SHEETS_TEMPLATE.format(
            **{"SHEET_TOKEN": sheet_token, "RANGE": "",}
        )
        return self.put(
            url, json={"valueRange": {"range": sheet_range, "values": value,}}
        )

    @post_request(larkDockError)
    def write_sheet_by_more_range(self, sheet_token, valueRanges):
        """向多个范围写入数据
        https://open.feishu.cn/document/ugTM5UjL4ETO14COxkTN/uEjMzUjLxIzM14SMyMTN

        Args:
            sheet_token (str): 表格的 token
            valueRanges ([type]): [description]

            values: 
                "valueRanges": [
                    {
                    "range": "range1",
                    "values": [
                        [
                        "string1", 1, "http://www.xx.com"
                        ]
                    ]
                    },
                    {
                    "range": "range2",
                    "values": [
                        [
                        "string2", 2, "http://www.xx.com"
                        ]
                    ]
                    }
                ]

        Returns:


            "responses": [
                {
                    "spreadsheetToken": "***",
                    "updatedCells": 0,
                    "updatedColumns": 0,
                    "updatedRange": "***",
                    "updatedRows": 0
                },
                {
                    "spreadsheetToken": "***",
                    "updatedCells": 0,
                    "updatedColumns": 0,
                    "updatedRange": "***",
                    "updatedRows": 0
                }
            ],
            "revision": 0,
            "spreadsheetToken": "***"
        """
        url = "https://open.feishu.cn/open-apis/sheet/v2/spreadsheets/{}/values_batch_update".format(
            sheet_token
        )
        return self.post(url, json={"valueRanges": valueRanges})

    @post_request(larkDockError)
    def lock_sheet(self, sheet_token, sheet_id, lock_info="Locked"):
        """锁定指定单元格

        Args:
            sheet_token (str): sheet_token
            sheet_id (str): sheet_id 子表的 id
            lock_info (str, optional):锁定的信息. Defaults to "Locked".
        """
        url = "https://open.feishu.cn/open-apis/sheet/v2/spreadsheets/{}/sheets_batch_update".format(
            sheet_token
        )
        return self.post(
            url,
            json={
                "requests": [
                    {
                        "updateSheet": {
                            "properties": {
                                "sheetId": sheet_id,
                                "protect": {"lock": "LOCK", "lockInfo": lock_info,},
                            }
                        }
                    }
                ]
            },
        )

    @post_request(larkDockError)
    def unlock_sheet(self, sheet_token, sheet_id):
        """解锁指定单元格

        Args:
            sheet_token (str): sheet_token
            sheet_id (str): sheet_id 子表的 id
        """
        url = "https://open.feishu.cn/open-apis/sheet/v2/spreadsheets/{}/sheets_batch_update".format(
            sheet_token
        )
        return self.post(
            url,
            json={
                "requests": [
                    {
                        "updateSheet": {
                            "properties": {
                                "sheetId": sheet_id,
                                "protect": {"lock": "UNLOCK",},
                            }
                        }
                    }
                ]
            },
        )

    @post_request(larkDockError)
    def get_root_folder_metadata(self):
        """获取 "我的空间" 的元信息
        https://open.feishu.cn/document/ugTM5UjL4ETO14COxkTN/uAjNzUjLwYzM14CM2MTN

        Returns:
            "data": {
                "token": "string",
                "id": "folderId",
                "user_id": "userId"
            }
        """
        url = "https://open.feishu.cn/open-apis/drive/explorer/v2/root_folder/meta"
        return self.get(url)



# TODO: 这里可以优化一下 不能全依赖过来
from .lark import *


class LarkRecallError(LarkError):
    pass


class LarkSender(Lark):
    """用于 Lark 的 各种消息发送
    https://open.feishu.cn/document/ukTMukTMukTM/ucDO1EjL3gTNx4yN4UTM
    """

    REVEIVERS_KINDS = ["open_id", "email", "chat_id", "user_id", "open_ids"]
    SEND_MESSAGE_URL = "https://open.feishu.cn/open-apis/message/v4/send/"
    RECALL_URL = "https://open.feishu.cn/open-apis/message/v4/recall/"
    READ_INFO_URL = "https://open.feishu.cn/open-apis/message/v4/read_info/"
    URGENT_URL = "https://open.feishu.cn/open-apis/message/v4/urgent/"

    def __init__(self, app_id, secret, receiver, root_id=None):
        super(LarkSender, self).__init__(app_id, secret)
        self.receiver = receiver
        self.root_id = root_id

    def genrate_msg(self, msg_type):
        # 复制一份防止修改原数据
        pre_msg_body = dict(**self.receiver)
        for key in self.receiver:
            # 去掉不合法的参数
            if key not in self.REVEIVERS_KINDS:
                del pre_msg_body[key]
        if self.root_id:
            pre_msg_body["root_id"] = self.root_id
        pre_msg_body["msg_type"] = msg_type
        return pre_msg_body

    def send_message(self, *args, **kwargs):
        return self.post(self.SEND_MESSAGE_URL, *args, **kwargs)

    @post_request(LarkRequestError)
    def send_text(self, message):
        """发送文本消息
        https://open.feishu.cn/document/ukTMukTMukTM/uUjNz4SN2MjL1YzM
        Args:
            message (text): 消息的内容

        Returns:
            message: 返回消息的 message_id

            {
                "message_id":str
            }
        """
        msg_body = self.genrate_msg("text")
        msg_body["content"] = {"text": message}
        return self.send_message(json=msg_body)

    @post_request(LarkRequestError)
    def send_post(self, title, content, language="zh_cn"):
        """发送富文本消息
        https://open.feishu.cn/document/ukTMukTMukTM/uMDMxEjLzATMx4yMwETM
        Args:
            title (str): 消息的 title
            content (str): 消息的正文
            language (str, optional): 语言类型，目前只支持同时发送一种，可选 ja_jp,en_us. Defaults to "zh_cn".
        
        Returns:
            message: 返回消息的 message_id

            {
                "message_id":str
            }
        """
        msg_body = self.genrate_msg("post")
        msg_body["content"] = {"post": {language: {"title": title, "content": content}}}
        return self.send_message(json=msg_body)

    @post_request(LarkRecallError, parse_func=lambda response: response)
    def recall_message(self, message_id):
        """撤回消息，注意必须要发送时长小于 24 h
        https://open.feishu.cn/open-apis/message/v4/recall/
        Args:
            message_id (str): 发送的消息 ID 

        Returns:
            response: 撤回结果

        {
            code:number
            msg:str
        }
        """
        return self.post(self.RECALL_URL, json={"message_id": message_id})

    @post_request(LarkRequestError)
    def read_info(self, message_id):
        """ 查看消息的已读情况 返回所有已读的消息列表

        Args:
            message_id (str): 消息的 message_id ID

        Returns:
            user_list: 已读的用户

            "read_users": [
                {
                    "open_id": "ou_18eac85d35a26f989317ad4f02e8bbbb",
                    "timestamp": "1570697776",
                    "user_id": "ca51d83b"
                }
            ]
        """
        return self.post(self.READ_INFO_URL, json={"message_id": message_id})

    @post_request(
        LarkRequestError,
        parse_func=lambda response: response["invalid_open_ids"]
        if "invalid_open_ids" in response
        else response,
    )
    def urgent(self, message_id, urgent_type="app"):
        """加急消息 需要额外申请权限

        https://open.feishu.cn/open-apis/message/v4/urgent/

        Args:
            message_id (str): 消息的 id
            urgent_type (str, optional): 加急的类型 应用内加急（app），短信加急（sms），电话加急（phone）。加急权限需要申请。. Defaults to "app".
        
        Returns:

            user_list: 用户列表
            [
                "ou_uj98af65g7910044998k064t3ghib90g"
            ]
        """
        open_ids = self.receiver.get("open_ids")
        if not open_ids:
            raise LarkvalueError("open_ids is None！")
        return self.post(
            self.URGENT_URL,
            json={
                "message_id": message_id,
                "urgent_type": urgent_type,
                "open_ids": open_ids,
            },
        )

    def send_card(self, card, update_multi=False):
        """发送卡片消息
        https://open.feishu.cn/document/ukTMukTMukTM/uYTNwUjL2UDM14iN1ATN
        
        Args:
            card (card): 要发送的卡片消息，详细数据结构请参考文档
            update_multi (bool, optional): 控制卡片是否是共享卡片(所有用户共享同一张消息卡片). Defaults to False.

        Returns:
            message_id: 消息 ID

            {
                "message_id": "om_92eb70a7120ga8c3ca56a12ee9ba7ca2"
            }
        """
        msg_body = self.genrate_msg("interactive")
        msg_body["update_multi"] = update_multi
        msg_body["card"] = card
        return self.send_message(msg_body)



#!/usr/bin/env python
# coding: utf-8
"""The setup script."""

from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

requirements = []

test_requirements = [
    "pytest>=3",
]

setup(
    author="holegots",
    author_email="gaopengyang@bytedance.com",
    python_requires=">=2.7,!=3.0.*,!=3.1.*",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    description="A python sdk that implements all the features of the official documentation (not implemented).",
    install_requires=requirements,
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",  # 指定包文档格式为markdown
    include_package_data=True,
    keywords="lark_sdk",
    name="lark_sdk",
    packages=find_packages(include=["lark_sdk", "lark_sdk.*"]),
    test_suite="tests",
    use_2to3=True,
    tests_require=test_requirements,
    url="https://github.com/fuergaosi233/lark_sdk",
    version="0.3.5",
    zip_safe=False,
)
